@@ -1,9 +1,9 @@
-# $Id: ODBC.pm 15016 2011-11-24 09:26:38Z mjevans $
+# $Id: ODBC.pm 15193 2012-03-02 11:09:04Z mjevans $
 #
 # Copyright (c) 1994,1995,1996,1998  Tim Bunce
 # portions Copyright (c) 1997-2004  Jeff Urlwin
 # portions Copyright (c) 1997  Thomas K. Wenrich
-# portions Copyright (c) 2007-2011 Martin J. Evans
+# portions Copyright (c) 2007-2012 Martin J. Evans
 #
 # You may distribute under the terms of either the GNU General Public
 # License or the Artistic License, as specified in the Perl README file.
@@ -19,7 +19,7 @@ require 5.008;
 # see discussion on dbi-users at
 # http://www.nntp.perl.org/group/perl.dbi.dev/2010/07/msg6096.html and
 # http://www.dagolden.com/index.php/369/version-numbers-should-be-boring/
-$DBD::ODBC::VERSION = '1.33';
+$DBD::ODBC::VERSION = '1.35';
 
 {
     ## no critic (ProhibitMagicNumbers ProhibitExplicitISA)
@@ -32,9 +32,9 @@ $DBD::ODBC::VERSION = '1.33';
 
     @ISA = qw(Exporter DynaLoader);
 
-    # my $Revision = substr(q$Id: ODBC.pm 15016 2011-11-24 09:26:38Z mjevans $, 13,2);
+    # my $Revision = substr(q$Id: ODBC.pm 15193 2012-03-02 11:09:04Z mjevans $, 13,2);
 
-    require_version DBI 1.21;
+    require_version DBI 1.609;
 
     bootstrap DBD::ODBC $VERSION;
 
@@ -43,6 +43,30 @@ $DBD::ODBC::VERSION = '1.33';
     $sqlstate = "00000";
     $drh = undef;       # holds driver handle once initialised
 
+    use constant {
+        # header fields in SQLGetDiagField:
+        SQL_DIAG_CURSOR_ROW_COUNT => -1249,
+        SQL_DIAG_DYNAMIC_FUNCTION => 7,
+        SQL_DIAG_DYNAMIC_FUNCTION_CODE => 12,
+        SQL_DIAG_NUMBER => 2,
+        SQL_DIAG_RETURNCODE => 1,
+        SQL_DIAG_ROW_COUNT => 3,
+        # record fields in SQLGetDiagField:
+        SQL_DIAG_CLASS_ORIGIN => 8,
+        SQL_DIAG_COLUMN_NUMBER => -1247,
+        SQL_DIAG_CONNECTION_NAME => 10,
+        SQL_DIAG_MESSAGE_TEXT => 6,
+        SQL_DIAG_NATIVE => 5,
+        SQL_DIAG_ROW_NUMBER => -1248,
+        SQL_DIAG_SERVER_NAME => 11,
+        SQL_DIAG_SQLSTATE => 4,
+        SQL_DIAG_SUBCLASS_ORIGIN => 9
+    };
+    our @EXPORT_DIAGS = qw(SQL_DIAG_CURSOR_ROW_COUNT SQL_DIAG_DYNAMIC_FUNCTION SQL_DIAG_DYNAMIC_FUNCTION_CODE SQL_DIAG_NUMBER SQL_DIAG_RETURNCODE SQL_DIAG_ROW_COUNT SQL_DIAG_CLASS_ORIGIN SQL_DIAG_COLUMN_NUMBER SQL_DIAG_CONNECTION_NAME SQL_DIAG_MESSAGE_TEXT SQL_DIAG_NATIVE SQL_DIAG_ROW_NUMBER SQL_DIAG_SERVER_NAME SQL_DIAG_SQLSTATE SQL_DIAG_SUBCLASS_ORIGIN);
+    our @EXPORT_OK = (@EXPORT_DIAGS);
+    our %EXPORT_TAGS = (
+        diags => \@EXPORT_DIAGS);
+
     sub parse_trace_flag {
         my ($class, $name) = @_;
         return 0x02_00_00_00 if $name eq 'odbcunicode';
@@ -56,23 +80,28 @@ $DBD::ODBC::VERSION = '1.33';
     }
 
     sub driver{
-	return $drh if $drh;
-	my($class, $attr) = @_;
+        return $drh if $drh;
+        my($class, $attr) = @_;
 
-	$class .= "::dr";
+        $class .= "::dr";
 
-	# not a 'my' since we use it above to prevent multiple drivers
+        # not a 'my' since we use it above to prevent multiple drivers
 
-	$drh = DBI::_new_drh($class, {
-	    'Name' => 'ODBC',
-	    'Version' => $VERSION,
-	    'Err'    => \$DBD::ODBC::err,
-	    'Errstr' => \$DBD::ODBC::errstr,
-	    'State' => \$DBD::ODBC::sqlstate,
-	    'Attribution' => 'DBD::ODBC by Jeff Urlwin, Tim Bunce and Martin J. Evans',
+        $drh = DBI::_new_drh($class, {
+            'Name' => 'ODBC',
+            'Version' => $VERSION,
+            'Err'    => \$DBD::ODBC::err,
+            'Errstr' => \$DBD::ODBC::errstr,
+            'State' => \$DBD::ODBC::sqlstate,
+            'Attribution' => 'DBD::ODBC by Jeff Urlwin, Tim Bunce and Martin J. Evans',
 	    });
         DBD::ODBC::st->install_method("odbc_lob_read");
-	return $drh;
+        # don't clear errors - IMA_KEEP_ERR = 0x00000004
+        DBD::ODBC::st->install_method("odbc_getdiagrec", { O=>0x00000004 });
+        DBD::ODBC::db->install_method("odbc_getdiagrec", { O=>0x00000004 });
+        DBD::ODBC::db->install_method("odbc_getdiagfield", { O=>0x00000004 });
+        DBD::ODBC::st->install_method("odbc_getdiagfield", { O=>0x00000004 });
+        return $drh;
     }
 
     sub CLONE { undef $drh }
@@ -148,7 +177,9 @@ $DBD::ODBC::VERSION = '1.33';
             odbc_putdata_start             => undef, # sth and dbh
             odbc_column_display_size       => undef, # sth and dbh
             odbc_utf8_on                   => undef, # sth and dbh
-	    odbc_driver_complete           => undef
+	    odbc_driver_complete           => undef,
+	    odbc_batch_size                => undef,
+            odbc_disable_array_operations  => undef, # sth and dbh
                };
     }
 
@@ -504,7 +535,9 @@ $DBD::ODBC::VERSION = '1.33';
             odbc_utf8_on                   => undef, # sth and dbh
             odbc_exec_direct               => undef, # sth and dbh
             odbc_old_unicode               => undef, # sth and dbh
-            odbc_describe_parameters => undef, # sth and dbh
+            odbc_describe_parameters       => undef, # sth and dbh
+	    odbc_batch_size                => undef, # sth and dbh
+            odbc_disable_array_operations  => undef, # sth and dbh
         };
     }
 
@@ -520,43 +553,72 @@ $DBD::ODBC::VERSION = '1.33';
 	return $tmp;
     }
 
-# Just in case someone comes along and wants to add this
-#    sub execute_for_fetch {
-#        my ($sth, $fetch_tuple_sub, $tuple_status) = @_;
-#        print "execute_for_fetch\n";
-#        my $row_count = 0;
-#        my $tuple_count="0E0";
-#        my $tuple_batch_status;
-#
-#        if (defined($tuple_status)) {
-#            @$tuple_status = ();
-#            $tuple_batch_status = [ ];
-#        }
-#        while (1) {
-#            my @tuple_batch;
-#            for (my $i = 0; $i < $batch_size; $i++) {
-#                push @tuple_batch, [ @{$fetch_tuple_sub->() || last} ];
-#            }
-#            last unless @tuple_batch;
-#            my $res = odbc_execute_array($sth,
-#                                         \@tuple_batch,
-#                                         scalar(@tuple_batch),
-#                                         $tuple_batch_status);
-#            if (defined($res) && defined($row_count)) {
-#                $row_count += $res;
-#            } else {
-#                $row_count = undef;
-#            }
-#            $tuple_count+=@$tuple_batch_status;
-#            push @$tuple_status, @$tuple_batch_status
-#                if defined($tuple_status);
-#        }
-#        if (!wantarray) {
-#            return undef if !defined $row_count;
-#            return $tuple_count;
-#        }
-#        return (defined $row_count ? $tuple_count : undef, $row_count);
-#    }
+    sub execute_for_fetch {
+        my ($sth, $fetch_tuple_sub, $tuple_status) = @_;
+        #print "execute_for_fetch\n";
+        my $row_count = 0;
+        my $tuple_count="0E0";
+        my $tuple_batch_status;
+        my $batch_size = $sth->FETCH('odbc_batch_size');
+
+        $sth->trace_msg("execute_for_fetch($fetch_tuple_sub, " .
+                            ($tuple_status ? $tuple_status : 'undef') .
+                                ") batch_size = $batch_size\n", 4);
+        # Use DBI's execute_for_fetch if ours is disabled
+        if ($sth->FETCH('odbc_disable_array_operations')) {
+            $sth->trace_msg("array operations disabled\n", 4);
+            my $sth = shift;
+            return $sth->SUPER::execute_for_fetch(@_);
+        }
+
+	$tuple_batch_status = [ ]; # we always want this here
+        if (defined($tuple_status)) {
+            @$tuple_status = ();
+        }
+        my $finished;
+        while (1) {
+            my @tuple_batch;
+            for (my $i = 0; $i < $batch_size; $i++) {
+                $finished = $fetch_tuple_sub->();
+                push @tuple_batch, [ @{$finished || last} ];
+            }
+            $sth->trace_msg("Found " . scalar(@tuple_batch) . " rows\n", 4);
+            last unless @tuple_batch;
+            my $res = odbc_execute_for_fetch($sth,
+					     \@tuple_batch,
+					     scalar(@tuple_batch),
+					     $tuple_batch_status);
+            $sth->trace_msg("odbc_execute_array returns " .
+                                ($res ? $res : 'undef') . "\n", 4);
+
+            #print "odbc_execute_array XS returned $res\n";
+            # count how many tuples were used
+            # basically they are all used unless marked UNUSED
+            if ($tuple_batch_status) {
+                foreach (@$tuple_batch_status) {
+                    $tuple_count++ unless $_ == 7; # SQL_PARAM_UNUSED
+                    next if ref($_);
+		    $_ = -1;	# we don't know individual row counts
+                }
+		if ($tuple_status) {
+		    push @$tuple_status, @$tuple_batch_status
+			if defined($tuple_status);
+		}
+            }
+            if (!defined($res)) {	# error
+                $row_count = undef;
+                last;
+            } else {
+                $row_count += $res;
+            }
+            last if !$finished;
+        }
+        if (!wantarray) {
+            return undef if !defined $row_count;
+            return $tuple_count;
+        }
+        return (defined $row_count ? $tuple_count : undef, $row_count);
+    }
 }
 
 1;
@@ -568,7 +630,7 @@ DBD::ODBC - ODBC Driver for DBI
 
 =head1 VERSION
 
-This documentation refers to DBD::ODBC version 1.33.
+This documentation refers to DBD::ODBC version 1.35.
 
 =head1 SYNOPSIS
 
@@ -927,6 +989,76 @@ situation was very specific since dates were being bound as dates when
 SQLDescribeParam was called and chars without and the data format was
 not a supported date format.
 
+=head2 Private methods common to connection and statement handles
+
+=head3 odbc_getdiagrec
+
+  @diags = $handle->odbc_getdiagrec($record_number);
+
+NOTE: This is an experimental method and may change.
+
+Introduced in 1.34_3.
+
+This is just a wrapper around the ODBC API SQLGetDiagRec. When a
+method on a connection or statement handle fails if there are any ODBC
+diagnostics you can use this method to retrieve them. Records start at
+1 and there may be more than 1. It returns an array containing the
+state, native and error message text or an empty array if the requested
+diagnostic record does not exist. To get all diagnostics available
+keep incrementing $record_number until odbc_getdiagrec returns an
+empty array.
+
+All of the state, native and message text are already passed to DBI
+via its set_err method so this method does not really tell you
+anything you cannot already get from DBI except when there is more
+than one diagnostic.
+
+You may find this useful in an error handler as you can get the ODBC
+diagnostics as they are and not how DBD::ODBC was forced to fit them
+into the DBI's system.
+
+NOTE: calling this method does not clear DBI's error values as usually
+happens.
+
+=head3 odbc_getdiagfield
+
+  $diag = $handle->odbc_getdiagfield($record, $identifier);
+
+NOTE: This is an experimental method and may change.
+
+This is just a wrapper around the ODBC API SQLGetDiagField. When a
+method on a connection or statement handle fails if there are any
+ODBC diagnostics you can use this method to retrieve the individual
+diagnostic fields. As with L</odbc_getdiagrec> records start at 1. The
+identifier is one of:
+
+  SQL_DIAG_CURSOR_ROW_COUNT
+  SQL_DIAG_DYNAMIC_FUNCTION
+  SQL_DIAG_DYNAMIC_FUNCTION_CODE
+  SQL_DIAG_NUMBER
+  SQL_DIAG_RETURNCODE
+  SQL_DIAG_ROW_COUNT
+  SQL_DIAG_CLASS_ORIGIN
+  SQL_DIAG_COLUMN_NUMBER
+  SQL_DIAG_CONNECTION_NAME
+  SQL_DIAG_MESSAGE_TEXT
+  SQL_DIAG_NATIVE
+  SQL_DIAG_ROW_NUMBER
+  SQL_DIAG_SERVER_NAME
+  SQL_DIAG_SQLSTATE
+  SQL_DIAG_SUBCLASS_ORIGIN
+
+DBD::ODBC exports these constants as 'diags' e.g.,
+
+  use DBD::ODBC qw(:diags);
+
+Of particular interest is SQL_DIAG_COLUMN_NUMBER as it will tell you
+which bound column or parameter is in error (assuming your driver
+supports it). See params_in_error in the examples dir.
+
+NOTE: calling this method does not clear DBI's error values as usually
+happens.
+
 =head2 Private connection attributes
 
 =head3 odbc_err_handler
@@ -1196,6 +1328,39 @@ As this only provides the driver and further attributes are required a
 dialogue will be thrown allowing you to specify the SQL Server to
 connect to and possibly other attributes.
 
+=head3 odbc_batch_size
+
+Sets the batch size for execute_for_fetch which defaults to 10.
+Bare in mind the bigger you set this the more memory DBD::ODBC will need
+to allocate when running execute_for_fetch and the memory required is
+max_length_of_pn * odbc_batch_size * n_parameters.
+
+=head3 odbc_disable_array_operations
+
+Please read the documentation on L<execute_array> and L<execute_for_fetch>
+which details subtle differences in DBD::ODBC's implementation compared
+with using DBI's default implementation. If these difference cause you
+a problem you can set odbc_disable_array_operations to true and DBD::ODBC
+will revert to DBI's implementations of the array methods.
+
+=head2 Private statement attributes
+
+=head3 odbc_more_results
+
+Use this attribute to determine if there are more result sets
+available.  SQL Server supports this feature.  Use this as follows:
+
+  do {
+     my @row;
+     while (@row = $sth->fetchrow_array()) {
+        # do stuff here
+     }
+  } while ($sth->{odbc_more_results});
+
+Note that with multiple result sets and output parameters (i.e,. using
+bind_param_inout), don't expect output parameters to be bound until ALL
+result sets have been retrieved.
+
 =head2 Private statement methods
 
 =head3 odbc_lob_read
@@ -1253,24 +1418,6 @@ the first lob, the second lob then return to the first lob. You must
 read all lobs in order and completely or read part of a lob and then
 do no further calls to odbc_lob_read.
 
-=head2 Private statement attributes
-
-=head3 odbc_more_results
-
-Use this attribute to determine if there are more result sets
-available.  SQL Server supports this feature.  Use this as follows:
-
-  do {
-     my @row;
-     while (@row = $sth->fetchrow_array()) {
-        # do stuff here
-     }
-  } while ($sth->{odbc_more_results});
-
-Note that with multiple result sets and output parameters (i.e,. using
-bind_param_inout), don't expect output parameters to be bound until ALL
-result sets have been retrieved.
-
 =head2 Private DBD::ODBC Functions
 
 You use DBD::ODBC private functions like this:
@@ -1476,6 +1623,8 @@ does this and without this flag you have to add 0 to all bound column
 data returning numbers to get JSON::XS to encode it is N instead of
 "N".
 
+NOTE: For StrictlyTyped you need at least DBI 1.611.
+
 =head3 StrictlyTyped
 
 See DBI's sql_type_cast utility function.
@@ -1491,6 +1640,8 @@ an SQL_INTEGER and the data is not able to be converted to an integer
 the ODBC driver will problably return "Invalid character value for
 cast specification (SQL-22018)".
 
+NOTE: For DiscardString you need at least DBI 1.611.
+
 =head3 TreatAsLOB
 
 See L</odbc_lob_read>.
@@ -1685,6 +1836,71 @@ cannot deviate from the specification but you should know it rolls
 back an uncommitted transaction when disconnect is called if
 SQLDisconnect returns state 25000 (transaction in progress).
 
+=head3 execute_for_fetch and execute_array
+
+From version 1.34_1 DBD::ODBC implements its own execute_for_fetch
+which binds arrays of parameters and can send multiple rows
+(L</odbc_batch_size>) of parameters through the ODBC driver in one go
+(this overrides DBI's default execute_for_fetch). This is much faster
+when inserting, updating or deleting many rows in one go. Note,
+execute_array uses execute_for_fetch when the parameters are passed
+for column-wise binding.
+
+However, there are a small number of differences between using
+DBD::ODBC's execute_for_fetch compared with using DBI's default
+implementation (which simply calls execute repeatedly once per row).
+The differences you may see are:
+
+o as DBI's execute_for_fetch does one row at a time the result from
+execute is for one row and just about all ODBC drivers can report the
+number of affected rows when SQLRowCount is called per execute. When
+batches of parameters are sent the driver can still return the number
+of affected rows but it is usually per batch rather than per row. As a
+result, the tuple_status array you may pass to execute_for_fetch (or
+execute_array) usually shows -1 (unknown) for each row although the total
+affected returned in array context is a correct total affected.
+
+o not all ODBC drivers have sufficient ODBC support (arguably a bug)
+for correct diagnostics support when using arrays. DBI dictates that
+if a row in the batch is in error the tuple_status will contain the
+state, native and error message text. However the batch may generate
+multiple errors per row (which DBI says nothing about) and more than
+one row may error. In ODBC we get a list of errors but to associate
+each one with a particular row we need to call SQLGetDiagField for
+SQL_DIAG_ROW_NUMBER and it should say which row in the batch the
+diagnostic is associated with. Some ODBC drivers do not support
+SQL_DIAG_ROW_NUMBER properly and then DBD::ODBC cannot know which row
+in the batch an error refers to. In this case DBD::ODBC will report an
+error saying "failed to retrieve diags", state of HY000 and a native
+of 1 so you'll still see an error but not necessarily the exact
+one. Also, when more than one diagnostic is found for a row DBD::ODBC
+picks the first one (which is usually most relevant) as there is no
+way to report more than one diagnostic per row in the tuple_status. If
+the first problem of SQL_DIAG_ROW_NUMBER proves to be a problem for
+you the DBD::ODBC tracing will show all errors and if that is still
+not sufficient I'd consider adding a method to show all diagnostics.
+
+o Binding parameters with execute_array and execute_for_fetch does not
+allow the parameter types to be set. However, as parameter types are sticky
+you can call bind_param(param_num, undef, {TYPE => sql_type}) before
+calling execute_for_fetch/execute_array and the TYPE should be sticky
+when the batch of parameters is bound.
+
+o Although you can insert very large columns execute_for_fetch will
+need L</odbc_batch_size> * max length of parameter per parameter so
+you may hit memory limits. If you use DBI's execute_for_fetch
+DBD::ODBC uses the ODBC API SQLPutData (see L</odbc_putdata_start>)
+which does not require large amounts of memory as large columns are
+sent in pieces.
+
+=head3 type_info_all
+
+Many ODBC drivers now return 20 columns in type_info_all rather than
+the 19 DBI documents DBI documents. The 20th column is usually called
+"USERTYPE".  Recent MS SQL Server ODBC drivers do this. Fortunately
+this should adversely affect you so long as you are using the keys
+provided at the start of type_info_all.
+
 =head2 Unicode
 
 The ODBC specification supports wide character versions (a postfix of
@@ -2021,7 +2237,7 @@ L<http://www.datadirect.com>
 
 L<http://www.atinet.com>
 
-Some useful tutorials:
+=head2 Some useful tutorials:
 
 Debugging Perl DBI:
 
@@ -2029,7 +2245,6 @@ L<http://www.easysoft.com/developer/languages/perl/dbi-debugging.html>
 
 Enabling ODBC support in Perl with Perl DBI and DBD::ODBC:
 
-
 L<http://www.easysoft.com/developer/languages/perl/dbi_dbd_odbc.html>
 
 Perl DBI/DBD::ODBC Tutorial Part 1 - Drivers, Data Sources and Connection:
@@ -2052,6 +2267,10 @@ Multiple Active Statements (MAS) and DBD::ODBC
 
 L<http://www.easysoft.com/developer/languages/perl/multiple-active-statements.html>
 
+64-bit ODBC
+
+L<http://www.easysoft.com/developer/interfaces/odbc/64-bit.html>
+
 =head2 Frequently Asked Questions
 
 Frequently asked questions are now in L<DBD::ODBC::FAQ>. Run
